#!/usr/bin/env python

'''
Feature homography
==================

Example of using features2d framework for interactive video homography matching.
ORB features and FLANN matcher are used. The actual tracking is implemented by
PlaneTracker class in plane_tracker.py

Inspired by http://www.youtube.com/watch?v=-ZNYoL8rzPY

video: http://www.youtube.com/watch?v=FirtmYcC0Vc

Usage
-----
feature_homography.py [<video source>]

Keys:
   SPACE  -  pause video

Select a textured planar object to track by drawing a box with a mouse.
'''

# Python 2/3 compatibility
from __future__ import print_function

# 导入所需库
import numpy as np
import cv2 as cv

# 导入其他模块
import video
from video import presets
import common
from common import getsize, draw_keypoints
from plane_tracker import PlaneTracker

class App:
    def __init__(self, src):
        # 创建视频捕获对象
        self.cap = video.create_capture(src, presets['book'])
        self.frame = None
        self.paused = False
        # 创建平面跟踪器对象
        self.tracker = PlaneTracker()

        # 创建全屏窗口
        cv.namedWindow('plane',cv.WND_PROP_FULLSCREEN)
        cv.setWindowProperty('plane', cv.WND_PROP_FULLSCREEN, cv.WINDOW_FULLSCREEN)

        # 创建矩形选择器对象，用于用户选择待跟踪的平面区域
        self.rect_sel = common.RectSelector('plane', self.on_rect)

    def on_rect(self, rect):
        # 当用户选择了一个矩形区域后的回调函数
        # 清除跟踪器中的所有目标
        self.tracker.clear()
        # 添加新的跟踪目标
        self.tracker.add_target(self.frame, rect)

    def run(self):
        while True:
            # 判断是否需要读取新的视频帧
            playing = not self.paused and not self.rect_sel.dragging
            if playing or self.frame is None:
                # 读取新的视频帧
                ret, frame = self.cap.read()
                if not ret:
                    break
                self.frame = frame.copy()
            # 创建用于显示的图像
            w, h = getsize(self.frame)
            vis = np.zeros((h*2, w, 3), np.uint8)
            vis[:h,:w] = self.frame

            # 如果有跟踪目标，显示目标的特征点和矩形框
            if len(self.tracker.targets) > 0:
                target = self.tracker.targets[0]
                vis[h:,:] = target.image
                draw_keypoints(vis[h:,:], target.keypoints)
                x0, y0, x1, y1 = target.rect
                cv.rectangle(vis, (x0, y0+h), (x1, y1+h), (0, 255, 0), 2)

            if playing:
                # 进行跟踪
                tracked = self.tracker.track(self.frame)
                if len(tracked) > 0:
                    # 绘制跟踪结果
                    tracked = tracked[0]
                    cv.polylines(vis, [np.int32(tracked.quad)], True, (255, 255, 255), 2)
                    for (x0, y0), (x1, y1) in zip(np.int32(tracked.p0), np.int32(tracked.p1)):
                        cv.line(vis, (x0, y0+h), (x1, y1), (0, 255, 0))

            # 绘制所有跟踪点
            draw_keypoints(vis, self.tracker.frame_points)

            # 绘制矩形选择框
            self.rect_sel.draw(vis)
            # 显示图像
            cv.imshow('plane', vis)

            ch = cv.waitKey(1)
            # 按下空格键暂停/播放
            if ch == ord(' '):
                self.paused = not self.paused
            # 按下ESC键退出
            if ch == 27:
                break


if __name__ == '__main__':
    print(__doc__)
    import sys
    try:
        video_src = sys.argv[1]
    except:
        video_src = 0

    # 运行程序
    App(video_src).run()
